-------------------------------------------------------------------------------
-- Addon namespace
local nanoTalk = LibStub("AceAddon-3.0"):NewAddon("nanoTalk", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
_G["nanoTalk"] = nanoTalk


local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
nanoTalk.dataobj = LDB:NewDataObject("nanoTalk", {
			type	= "data source",
			icon	= "Interface\\GossipFrame\\GossipGossipIcon",
			label	= "nanoTalk",
			text	= "0t",
		})


-------------------------------------------------------------------------------
-- Localized globals
local ipairs, pairs = ipairs, pairs
local tremove, tinsert, tsort = table.remove, table.insert, table.sort


-------------------------------------------------------------------------------
-- Local variables
local version = GetAddOnMetadata("nanoTalk", "Version")
local db
local defaults = {
	profile = {
		ChatNotify = true,
		NotifyAFK = true,
		SoundAlert = true,
		TalkTime = 2,
		TrackNum = 10,
		TrackTime = 120,
		Version = version
	},
	factionrealm = {
		History = {}
	},
}
local trackSecs, talkSecs
local timers = {}
local updatedLines = {}
local timestamps = {}
local people = {}
local AFKtime = 0
local tooltip = LibStub:GetLibrary("LibQTip-1.0"):Acquire("nanoTalk", 3, "LEFT", "CENTER", "CENTER")


-------------------------------------------------------------------------------
-- Local functions
local function SetTimes()
	trackSecs = (db.TrackTime * 60)
	talkSecs = (db.TalkTime * 60)
end


local function GetCount(timer)
	if (timer == nil) then
		return #timestamps
	end

	local count = 0

	for _, tm in ipairs(timestamps) do
		if ((GetTime() - tm) <= timer) then
			count = count + 1
		end
	end
	return count
end


local function TimeStr(seconds)
	local tm = seconds
	local timeval = tostring(math.floor(tm % 60)).."s"

	if (tm >= 60) then
		tm = tm / 60
		timeval = tostring(math.floor(tm % 60)).."m"..timeval

		if (tm >= 60) then
			tm = tm / 60
			timeval = tostring(math.floor(tm % 24)).."h"..timeval

			if (tm >= 24) then
				tm = tm / 24
				timeval = tostring(math.floor(tm)).."d"..timeval
			end
		end
	end
	return timeval
end


local function ShowTooltip()
	tooltip:Clear()
	tooltip:AddHeader("nanoTalk")
	tooltip:AddLine(" ")

	local count = GetCount(trackSecs)
	local linenum

	if (count > 0) then
		tooltip:AddHeader("|cffff9933Name|r", "|cffff9933Messages|r", "|cffff9933Last|r")

		for _, tm in ipairs(timestamps) do
			local curtm = GetTime() - tm

			if (curtm <= trackSecs) then
				linenum = tooltip:AddLine(people[tm].name, people[tm].msgs, TimeStr(curtm))
				updatedLines[linenum] = tm
			end
		end
		tooltip:AddLine(" ")
		linenum = tooltip:AddLine("Left Click:")
		tooltip:SetCell(linenum, 2, "Clear list.")
	else
		tooltip:AddLine("No messages.")
		tooltip:AddLine(" ")
	end
	linenum = tooltip:AddLine("Right Click:")
	tooltip:SetCell(linenum, 2, "Configure.")
	tooltip:Show()
end


local function UpdateTooltip()
	if (tooltip:IsShown()) then
		updatedLines = {}
		ShowTooltip()
	end
end


-------------------------------------------------------------------------------
-- LDB functions
function nanoTalk.dataobj:OnClick(button)
	if (button == "LeftButton") then
		nanoTalk:Clear()
	else
		InterfaceOptionsFrame_OpenToCategory(nanoTalk.optionsFrames.nanoTalk)
	end
end


function nanoTalk.dataobj.OnLeave()
	tooltip:Hide()
	updatedLines = {}
end


function nanoTalk.dataobj.OnEnter(self)
	tooltip:SmartAnchorTo(self)
	ShowTooltip()
end


-------------------------------------------------------------------------------
-- Initialization and event handlers
function nanoTalk:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("nanoTalkConfig", defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db = self.db.profile

	if not self.dataobj then self.dataobj = {} end

	SetTimes()
--	self:CreateFrames()
	self:SetupOptions()
end


function nanoTalk:OnEnable()
	self:RegisterEvent("CHAT_MSG_WHISPER")
	timers.UpdateInfo = self:ScheduleRepeatingTimer("UpdateInfo", 0.5)
end


function nanoTalk:OnDisable()
	self:UnregisterEvent("CHAT_MSG_WHISPER")

	for k, v in pairs(timers) do
		self:CancelTimer(v, true)
		timers[k] = nil
	end
end


function nanoTalk:OnProfileChanged(event, database, newProfileKey)
	db = database.profile
end


function nanoTalk:CHAT_MSG_WHISPER(event, msg, player)
	local stamp = GetTime()
	local last = nil

	for tm, plr in pairs(people) do
		if (plr.name == player) then
			last = tm
			break
		end
	end

	local msgcnt = 0

	if (last ~= nil) then
		msgcnt = people[last].msgs
		people[last] = nil

		for idx, tm in ipairs(timestamps) do
			if (tm == last) then
				tremove(timestamps, idx)
				break
			end
		end
	end

	people[stamp] = {
		name = player,
		msgs = msgcnt + 1
	}
	tinsert(timestamps, stamp)

	if (#timestamps > 1) then tsort(timestamps) end

	if db.SoundAlert then PlaySoundFile("Interface\\AddOns\\nanoTalk\\beep.wav") end

	if ((db.NotifyAFK == true) and UnitIsAFK("player")) then
		SendChatMessage("I have been AFK for "..TimeStr(GetTime() - AFKtime)..".", "WHISPER", nil, player)
	elseif ((last == nil) or ((stamp - last) > talkSecs)) then
		local count = GetCount(talkSecs) - 1

		if (count < 1) then UpdateTooltip() return end

		if (db.ChatNotify == true) then
			local phrase = "Someone else is"

			if (count > 1) then
				phrase = "I have "..count.." other people"
			end

			SendChatMessage("Hello, "..player.."! "..phrase.." talking to me, so I might not answer right away.", "WHISPER", nil, player)
			self:Print(ChatFrame1, "Informed "..player.." of conversation.")
		end
	end
	UpdateTooltip()
end


function nanoTalk:UpdateInfo()
	if (#timestamps > db.TrackNum) then
		local ppos = tremove(timestamps)
		people[ppos] = nil
	end

	if (AFKtime == 0) then
		if (UnitIsAFK("player")) then
			AFKtime = GetTime()
		end
	elseif (not UnitIsAFK("player")) then
		self:Print(ChatFrame1, "You were AFK for "..TimeStr(GetTime() - AFKtime)..".")
		AFKtime = 0
	end

	local total = GetCount(trackSecs)
	local recent = GetCount(talkSecs)

	if (recent > 0) then
		self.dataobj.text = ""..total.."t "..recent.."r"
		self.dataobj.icon = "Interface\\GossipFrame\\PetitionGossipIcon"
	else
		self.dataobj.text = ""..total.."t "
		self.dataobj.icon = "Interface\\GossipFrame\\GossipGossipIcon"
	end

	if (tooltip:IsShown() and #updatedLines) then
		for linenum,tm in pairs(updatedLines) do
			tooltip:SetCell(linenum, 3, TimeStr(GetTime() - tm))
		end
	end
end


function nanoTalk:Clear()
	timestamps = {}
	people = {}
	UpdateTooltip()
end


-------------------------------------------------------------------------------
-- Configuration
local ACD = LibStub("AceConfigDialog-3.0")


local function GetOptions()
	if not nanoTalk.options then
		nanoTalk.options = {
			type = "group",
			name = "nanoTalk",
			args = {
				talktime = {
					type = "range",
					name = "Conversation Timer",
					desc = "How many minutes you will be considered to be in a conversation.",
					width = "full",
					min = 1,
					max = 10,
					step = 1,
					order = 100,
					get = function() return db.TalkTime end,
					set = function(info, v)
						      db.TalkTime = v
						      SetTimes()
					      end
				},
				tracktime = {
					type = "range",
					name = "Tracking Timer",
					desc = "How many minutes to track people who have sent you a message.",
					width = "full",
					min = 1,
					max = 120,
					step = 1,
					order = 110,
					get = function() return db.TrackTime end,
					set = function(info, v)
						      db.TrackTime = v
						      SetTimes()
					      end
				},
				tracknum = {
					type = "range",
					name = "People to Track",
					desc = "Maximum number of people to track.",
					width = "full",
					min = 0,
					max = 50,
					step = 1,
					order = 120,
					get = function() return db.TrackNum end,
					set = function(info, v) db.TrackNum = v end
				},
				chatnotify = {
					type = "toggle",
					name = "Chat Notification",
					desc = "Notify others when you are in a conversation.",
					order = 130,
					get = function() return db.ChatNotify end,
					set = function(info, v) db.ChatNotify = v end
				},
				afknotify = {
					type = "toggle",
					name = "AFK Notification",
					desc = "Notify others when you are AFK.",
					order = 140,
					get = function() return db.NotifyAFK end,
					set = function(info, v) db.NotifyAFK = v end
				},
				soundalert = {
					type = "toggle",
					name = "Sound Alert",
					desc = "Beep when receiving a message.",
					order = 150,
					get = function() return db.SoundAlert end,
					set = function(info, v) db.SoundAlert = v end
				},
			},
		}
	end
	return nanoTalk.options
end


function nanoTalk:SetupOptions()
	self.optionsFrames = {}

	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("nanoTalk", GetOptions())
	LibStub("AceConfig-3.0"):RegisterOptionsTable("nanoTalk", GetOptions(), "nanotalk")

	self.optionsFrames.nanoTalk = ACD:AddToBlizOptions("nanoTalk")
	LibStub("tekKonfig-AboutPanel").new("nanoTalk", "nanoTalk")
end
